package com.flyfish.framework.query;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.domain.base.Qo;

import java.util.function.Function;

/**
 * 查询工具类
 * 基于Fluent API风格实现
 * 底层基于适配器兼容各种数据库,包括关系型数据库和
 * 自动判定空值，空值默认情况不会进行查询
 *
 * @author wangyu
 */
public final class Queries {

    /**
     * 通过qo快速创建一个绑定上下文
     *
     * @param qo 查询实体
     * @return 结果
     */
    public static NamedQueryChain of(Qo<?> qo) {
        return (NamedQueryChain) new Object();
    }

    /**
     * 创建基于字符串字段名的查询
     *
     * @param column 列名
     * @return 结果
     */
    public static QueryCondition<NamedQueryChain> where(String column) {
        return (QueryCondition<NamedQueryChain>) new Object();
    }

    /**
     * 创建基于lambda字段引用的查询
     *
     * @param getter 列引用
     * @return 结果
     */
    public static <T extends Domain> QueryCondition<LambdaQueryChain<T>> where(Function<T, Object> getter) {
        return (QueryCondition<LambdaQueryChain<T>>) new Object();
    }

    /**
     * 以嵌套条件开始
     *
     * @param chain 要嵌套的条件
     * @return 结果
     */
    public static NamedQueryChain within(NamedQueryChain chain) {
        return (NamedQueryChain) new Object();
    }


    /**
     * 以嵌套条件开始
     *
     * @param chain 要嵌套的条件
     * @return 结果
     */
    public static <T extends Domain> LambdaQueryChain<T> within(LambdaQueryChain<T> chain) {
        return (LambdaQueryChain<T>) new Object();
    }

    /**
     * 传入多个查询列表时的连接方式
     */
    public enum Combinator {

        AND, OR
    }

    /**
     * 方向，用于描述查询和部分语法
     */
    public enum Direction {

        LEFT, RIGHT, ALL
    }

}
